@@ -123,65 +123,169 @@ brltty_brl_close_device ()
     brlapi_closeConnection();
 }
 
+static void
+send_key (gint layer,
+          gint key)
+{
+    BRLEventData bed;
+    sprintf(&dd.key_codes[0], "L%02dK%02d", layer, key);
+    bed.key_codes = dd.key_codes;						
+    client_callback(BRL_EVCODE_KEY_CODES, &bed);
+}
+
 static gboolean
 brltty_brl_glib_cb (GIOChannel   *source, 
 		    GIOCondition condition, 
 		    gpointer     data)
 {
-    brl_keycode_t	keypress;
+    brl_keycode_t	key;
 
     BRLEventCode 	bec;
     BRLEventData 	bed;
 
-    while (brlapi_readKey (0, &keypress) == 1) 
-    {
-	/* TODO: Find a better way to map brltty commands to gnopernicus keys. */
-	switch (keypress & ~BRL_FLG_TOGGLE_MASK) 
-	{
-	    case BRL_CMD_LNUP:
-    		sprintf(&dd.key_codes[0], "DK00");
-    		bec = BRL_EVCODE_KEY_CODES;
-    		bed.key_codes = dd.key_codes;						
-    		client_callback (bec, &bed);
-    	    break;
-      
-	    case BRL_CMD_HOME:
-    		sprintf(&dd.key_codes[0], "DK01");
-    		bec = BRL_EVCODE_KEY_CODES;
-    		bed.key_codes = dd.key_codes;						
-    		client_callback (bec, &bed);
-    	    break;
+    gint result;
 
+    while ((result = brlapi_readKey (0, &key)) == 1) 
+    {
+        /* This code supports older versions of BrlAPI which don't support the
+         * autorepeat feature. It ensures that all of the autorepeat flags are
+         * cleared and makes this client behave as though the feature has been
+         * turned off.
+         */
+        if (key & BRL_FLG_REPEAT_INITIAL) {
+            /* Key press for a command which is to begin repeating immediately.
+             * The corresponding key release will be a no-op so Just get rid of
+             * all the repeat flags.
+             */
+            key &= ~(BRL_FLG_REPEAT_INITIAL | BRL_FLG_REPEAT_DELAY);
+        } else if (key & BRL_FLG_REPEAT_DELAY) {
+            /* Key press for a command which is to begin repeating aftger a
+             * delay. The corresponding key release will be the very same
+             * command with no repeat flags set so just ignore this key press.
+             */
+            continue;
+        }
+
+
+	/* This is an attempt to match as many BRLTTY commands
+	 * to Gnopernicus equivalents as possible.
+	 * Not all drivers will send all commands.
+	 * If the user undefines the Gnopernicus key, the BRLTTY
+	 * command will stop working.
+	 */
+	switch (key & BRL_MSK_CMD) 
+	{
+	    /*goto title*/
+	    case BRL_CMD_TOP_LEFT:
+		send_key(0, 7);
+		break;
+
+	    /*goto statusbar*/
+	    case BRL_CMD_BOT_LEFT:
+		send_key(0, 3);
+		break;
+
+	    /*goto menu*/
+	    case BRL_CMD_TOP:
+		send_key(0, 9);
+		break;
+
+	    /*goto toolbar*/
+	    case BRL_CMD_BOT:
+		send_key(0, 1);
+		break;
+
+	    /*goto focus*/
+	    case BRL_CMD_HOME:
+	    case BRL_CMD_RETURN:
+		send_key(0, 12);
+		break;
+
+	    /*goto parent*/
+	    case BRL_CMD_PRPROMPT:
+		send_key(0, 8);
+		break;
+
+	    /*goto child*/
+	    case BRL_CMD_NXPROMPT:
+		send_key(0, 2);
+		break;
+
+	    /*goto first*/
+	    case BRL_CMD_PRPGRPH:
+		send_key(0, 15);
+		break;
+
+	    /*goto last*/
+	    case BRL_CMD_NXPGRPH:
+		send_key(0, 14);
+		break;
+
+	    /*goto previous*/
+	    case BRL_CMD_LNUP:
+	    case BRL_CMD_PRDIFLN:
+		send_key(0, 4);
+		break;
+	    
+	    /*goto next*/
 	    case BRL_CMD_LNDN:
-    		sprintf(&dd.key_codes[0], "DK02");
-    		bec = BRL_EVCODE_KEY_CODES;
-    		bed.key_codes = dd.key_codes;						
-    		client_callback (bec, &bed);
-    	    break;
-      
+	    case BRL_CMD_NXDIFLN:
+		send_key(0, 6);
+		break;
+
+	    /*toggle FocusTracking/FlatReview mode*/
+	    case BRL_CMD_FREEZE:
+		send_key(0, 10);
+		break;
+
+	    /*display left*/
 	    case BRL_CMD_FWINLT:
 	    case BRL_CMD_FWINLTSKIP:
-    		sprintf(&dd.key_codes[0], "DK03");
-    		bec = BRL_EVCODE_KEY_CODES;
-    		bed.key_codes = dd.key_codes;						
-    		client_callback (bec, &bed);
-    	    break;
-      
+    		send_key(9, 4);
+		break;
+
+	    /*display right*/
 	    case BRL_CMD_FWINRT:
 	    case BRL_CMD_FWINRTSKIP:
-    		sprintf(&dd.key_codes[0], "DK05");
-        	bec = BRL_EVCODE_KEY_CODES;
-    		bed.key_codes = dd.key_codes;						
-    		client_callback (bec, &bed);
-    	    break;
-      
-    	    /* TBD: Default action (DK01DK02) and repeat last */
-	    default: 
-	    {
-    		gint key = keypress & BRL_MSK_BLK;
-    		gint arg = keypress & BRL_MSK_ARG;
+		send_key(9, 6);
+		break;
 
-    		switch (key) 
+	    /*char left*/
+	    case BRL_CMD_CHRLT:
+		send_key(9, 1);
+		break;
+
+	    /*char right*/
+	    case BRL_CMD_CHRRT:
+		send_key(9, 3);
+		break;
+
+	    /*decrease speech volume*/
+	    case BRL_CMD_SAY_SOFTER:
+		send_key(8, 1);
+		break;
+
+	    /*increase speech volume*/
+	    case BRL_CMD_SAY_LOUDER:
+		send_key(8, 3);
+		break;
+		    
+	    /*decrease speech rate*/
+	    case BRL_CMD_SAY_SLOWER:
+		send_key(8, 4);
+		break;
+
+	    /*increase speech rate*/
+	    case BRL_CMD_SAY_FASTER:
+		send_key(8, 6);
+		break;
+		    
+	    default:
+	    {
+    		gint blk = key & BRL_MSK_BLK;
+    		gint arg = key & BRL_MSK_ARG;
+
+    		switch (blk) 
     		{
     		    case BRL_BLK_ROUTE:
 			sprintf(&dd.sensor_codes[0], "HMS%02d", arg);
@@ -190,14 +294,19 @@ brltty_brl_glib_cb (GIOChannel   *source,
 			client_callback (bec, &bed);
 		    break;
 		    default:
-			fprintf(stderr,_("BRLTTY command code not bound to Gnopernicus key code: 0X%04X\n"), keypress);
+			fprintf(stderr,_("BRLTTY command code not bound to Gnopernicus key code: 0X%04X\n"), key);
 		    break;
     		}
-    	    break;
 	    }
+	    break;
+	    case BRL_CMD_NOOP:
+		break;
 	}
     }
-    return TRUE;
+    if (result == -1)
+	brlapi_perror("brlapi_readKey");
+
+    return result != -1;
 }
 
 static void
@@ -209,36 +318,9 @@ ignore_block (gint block)
     }
 }
 
-static void
-ignore_input (gint block)
-{
-    static const gint flags[] = {
-    		         0 |		      0 |          	  0 |           	 0,
-	                 0 |         	      0 |        	  0 | BRL_FLG_CHAR_CONTROL,
-    		         0 |         	      0 | BRL_FLG_CHAR_META |           	 0,
-	                 0 |         	      0 | BRL_FLG_CHAR_META | BRL_FLG_CHAR_CONTROL,
-            		 0 | BRL_FLG_CHAR_UPPER |        	  0 |           	 0,
-    		         0 | BRL_FLG_CHAR_UPPER |        	  0 | BRL_FLG_CHAR_CONTROL,
-	                 0 | BRL_FLG_CHAR_UPPER | BRL_FLG_CHAR_META |            	 0,
-            		 0 | BRL_FLG_CHAR_UPPER | BRL_FLG_CHAR_META | BRL_FLG_CHAR_CONTROL,
-        BRL_FLG_CHAR_SHIFT |         	      0 |        	  0 |           	 0,
-        BRL_FLG_CHAR_SHIFT |         	      0 |        	  0 | BRL_FLG_CHAR_CONTROL,
-        BRL_FLG_CHAR_SHIFT |         	      0 | BRL_FLG_CHAR_META |           	 0,
-        BRL_FLG_CHAR_SHIFT |   		      0 | BRL_FLG_CHAR_META | BRL_FLG_CHAR_CONTROL,
-        BRL_FLG_CHAR_SHIFT | BRL_FLG_CHAR_UPPER |        	  0 |           	 0,
-        BRL_FLG_CHAR_SHIFT | BRL_FLG_CHAR_UPPER |        	  0 | BRL_FLG_CHAR_CONTROL,
-        BRL_FLG_CHAR_SHIFT | BRL_FLG_CHAR_UPPER | BRL_FLG_CHAR_META |           	 0,
-        BRL_FLG_CHAR_SHIFT | BRL_FLG_CHAR_UPPER | BRL_FLG_CHAR_META | BRL_FLG_CHAR_CONTROL
-    };
-    const gint *flag = flags + (sizeof(flags) / sizeof(*flag));
-    do {
-        ignore_block(block | *--flag);
-    } while (*flag);
-}
-
 gint
 brltty_brl_open_device (gchar* 			device_name, 
-			gshort 			port,
+			const gchar 		*port,
 			BRLDevCallback 	        device_callback, 
 			BRLDevice 		*device)
 {
@@ -246,7 +328,7 @@ brltty_brl_open_device (gchar* 			device_name,
     brlapi_settings_t settings;
     GdkWindow *root;
     GdkAtom VTAtom, type;
-    guint VT = 0;
+    guint VT = -1;
     gint format, length;
     unsigned char *buf;
 
@@ -291,7 +373,7 @@ brltty_brl_open_device (gchar* 			device_name,
      This only works with XFree86 starting from 4.3.99.903 and X.Org
      starting from 6.8.0
 
-     If you have an older version, the tty will be kept to 0 so that the
+     If you have an older version, the tty will be kept to -1 so that the
      brlapi_getTty() function will look at the CONTROLVT environment variable.
      So people having old versions should have something like this in their
      .xsession, before launching gnopernicus:
@@ -351,8 +433,8 @@ gettty:
 	return 0;
     }
 
-    ignore_input(BRL_BLK_PASSCHAR);
-    ignore_input(BRL_BLK_PASSDOTS);
+    ignore_block(BRL_BLK_PASSCHAR);
+    ignore_block(BRL_BLK_PASSDOTS);
     ignore_block(BRL_BLK_PASSKEY);
 
     return 1;
