@@ -1,7 +1,7 @@
 /* mag.c
  *
- * Copyright 2001, 2002 Sun Microsystems, Inc.,
- * Copyright 2001, 2002 BAUM Retec, A.G.
+ * Copyright 2001 - 2005 Sun Microsystems, Inc.,
+ * Copyright 2001 - 2005 BAUM Retec, A.G.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +23,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>/*4 sleep only*/
+#include <gdk/gdk.h>
 #include "mag_ctrl.h"
 #include "mag.h"
 #include "magxmlapi.h"
@@ -84,8 +85,8 @@ int      	crosswire_size  = 0    ;/*thickness of crosswire cursor*/
 guint32		crosswire_color = 0    ;/*color as AlphaRedGreenBlue*/
 gboolean	crosswire_clip  = FALSE;/*whenever to insert the cursor over the crosswire or not*/
 
+MagRectangle display_size;
 MagRectangle source_rect;
-MagRectangle target_rect;
 static MagRectangle rect;
 static MagRectangle mouse_tracked_roi;
 
@@ -566,10 +567,8 @@ mag_zoomer_set_ROI_left (MagZoomer *mag_zoomer,
 
 	if (check_for_focus_tracking_none (mag_zoomer))
 	    roi_left = mouse_tracked_roi.left;
-	if ( roi_left > source_rect.left)
-	    mag_zoomer->region_of_interest.left = roi_left;
-	else
-	    mag_zoomer->region_of_interest.left = source_rect.left;
+
+        mag_zoomer->region_of_interest.left = roi_left;
 }	
 
 void 
@@ -580,10 +579,8 @@ mag_zoomer_set_ROI_top (MagZoomer *mag_zoomer,
 
 	if (check_for_focus_tracking_none (mag_zoomer))
 	    roi_top = mouse_tracked_roi.top;
-	if ( roi_top > source_rect.top)
-	    mag_zoomer->region_of_interest.top = roi_top;
-	else
-	    mag_zoomer->region_of_interest.top = source_rect.top;	
+        
+	mag_zoomer->region_of_interest.top = roi_top;
 	
 }
 
@@ -595,10 +592,8 @@ mag_zoomer_set_ROI_width (MagZoomer *mag_zoomer,
 	
 	if (check_for_focus_tracking_none (mag_zoomer))
 	    roi_width = mouse_tracked_roi.width;
-	if ( roi_width < source_rect.width)
-	    mag_zoomer->region_of_interest.width = roi_width;
-	else
-	    mag_zoomer->region_of_interest.width = source_rect.width; 	
+        
+	mag_zoomer->region_of_interest.width = roi_width;
 }
 
 void 
@@ -612,20 +607,43 @@ mag_zoomer_set_ROI_height (MagZoomer *mag_zoomer,
 	    roi_height = mouse_tracked_roi.height;
 	    zoom_factor_was_changed = FALSE;
 	}
-	if ( roi_height < source_rect.height)
-    	    mag_zoomer->region_of_interest.height = roi_height;
-	else
-    	    mag_zoomer->region_of_interest.height = source_rect.height;	
+        
+	mag_zoomer->region_of_interest.height = roi_height;
 }
 
+
 /* __ MAG ZOOMER ZP METHODS ____________________________________________________*/
+void 
+mag_zoomer_set_ZP_extents (MagZoomer *mag_zoomer)
+{
+    MagRectangle temp_rect;
+
+    zoom_factor_changed = TRUE;
+    
+    magnifier_set_target (magnifier,
+			 &((mag_zoomer->zoomer_placement).extents));
+
+    /* get_source has to be called every time set_target is called , 
+    because the source changes*/			 
+    magnifier_get_source (magnifier,
+			      &source_rect);
+			 
+    temp_rect.left = temp_rect.top = 0;
+    temp_rect.width = mag_zoomer->zoomer_placement.extents.width - mag_zoomer->zoomer_placement.extents.left;
+    temp_rect.height = mag_zoomer->zoomer_placement.extents.height - mag_zoomer->zoomer_placement.extents.top;
+
+    magnifier_resize_region (magnifier,
+			     0,
+			     &temp_rect);			    
+}
+
 void 
 mag_zoomer_set_ZP_extents_left (MagZoomer *mag_zoomer, 
 				gchar     *ZPLeft)
 {
     MagRectangle temp_rect;
     int zp_left;
-    
+
     if (!ZPLeft)
 	return;
 
@@ -635,13 +653,15 @@ mag_zoomer_set_ZP_extents_left (MagZoomer *mag_zoomer,
     
     zoom_factor_changed = TRUE;
     
-    if ( zp_left > target_rect.left)
+    if ( zp_left > display_size.left)
 	(mag_zoomer->zoomer_placement).extents.left = zp_left;
     else
-	(mag_zoomer->zoomer_placement).extents.left = target_rect.left;    
+	(mag_zoomer->zoomer_placement).extents.left = display_size.left;    
 
-    magnifier_set_target (magnifier,
-			 &((mag_zoomer->zoomer_placement).extents));
+    /* get_source has to be called every time set_target is called , 
+    because the source changes*/			 
+    magnifier_get_source (magnifier,
+			      &source_rect);
 			 
     temp_rect.left = temp_rect.top = 0;
     temp_rect.width = mag_zoomer->zoomer_placement.extents.width - mag_zoomer->zoomer_placement.extents.left;
@@ -667,13 +687,14 @@ void mag_zoomer_set_ZP_extents_top (MagZoomer *mag_zoomer,
     
     zoom_factor_changed = TRUE;
 
-    if ( zp_top > target_rect.top)
+    if ( zp_top > display_size.top)
 	mag_zoomer->zoomer_placement.extents.top = zp_top;
     else
-	mag_zoomer->zoomer_placement.extents.top = target_rect.top;    
+	mag_zoomer->zoomer_placement.extents.top = display_size.top;    
 
-    magnifier_set_target (magnifier,
-			 &((mag_zoomer->zoomer_placement).extents));
+			 
+    magnifier_get_source (magnifier,
+			      &source_rect);
 
     temp_rect.left = temp_rect.top = 0;
     temp_rect.width = mag_zoomer->zoomer_placement.extents.width - mag_zoomer->zoomer_placement.extents.left;
@@ -700,13 +721,13 @@ mag_zoomer_set_ZP_extents_width (MagZoomer *mag_zoomer,
 
     zoom_factor_changed = TRUE;
 
-    if ( zp_width < target_rect.width)
+    if ( zp_width < display_size.width)
         mag_zoomer->zoomer_placement.extents.width = zp_width;
     else
-        mag_zoomer->zoomer_placement.extents.width = target_rect.width;	
+        mag_zoomer->zoomer_placement.extents.width = display_size.width;	
 
-    magnifier_set_target (magnifier,
-			 &((mag_zoomer->zoomer_placement).extents));
+    magnifier_get_source (magnifier,
+			      &source_rect);
 
     temp_rect.left = temp_rect.top = 0;
     temp_rect.width = mag_zoomer->zoomer_placement.extents.width - mag_zoomer->zoomer_placement.extents.left;
@@ -723,7 +744,7 @@ mag_zoomer_set_ZP_extents_height (MagZoomer *mag_zoomer,
 {
     MagRectangle temp_rect;
     int zp_height;
-    
+
     if (!ZPHeight)
 	return;
 
@@ -733,13 +754,14 @@ mag_zoomer_set_ZP_extents_height (MagZoomer *mag_zoomer,
     
     zoom_factor_changed = TRUE;
     	
-    if ( zp_height < target_rect.height)
+    if ( zp_height < display_size.height)
         mag_zoomer->zoomer_placement.extents.height = zp_height;
     else
-        mag_zoomer->zoomer_placement.extents.height = target_rect.height;	    
+        mag_zoomer->zoomer_placement.extents.height = display_size.height;	    
 
-    magnifier_set_target (magnifier,
-			 &((mag_zoomer->zoomer_placement).extents));
+    
+    magnifier_get_source (magnifier,
+			      &source_rect);
 
     temp_rect.left = temp_rect.top = 0;
     temp_rect.width = mag_zoomer->zoomer_placement.extents.width - mag_zoomer->zoomer_placement.extents.left;
@@ -901,11 +923,21 @@ mag_zoomers_set_cursor_color (gchar *color)
 	{
 	    magnifier_set_cursor_color (magnifier,
 					cursor_color);
-	    magnifier_set_cursor (magnifier,
+	    if (cursor_scale)
+	    {
+		    magnifier_set_cursor (magnifier,
+				  cursor_name, 
+				  CURSOR_SIZE_INVALID, 
+				  cursor_zoom_factor);
+	    }
+	    else
+	    {
+		magnifier_set_cursor (magnifier,
 				  cursor_name, 
 				  cursor_size, 
 				  cursor_zoom_factor);
-
+	    }
+	    
 	}
     }
 }
@@ -1464,6 +1496,7 @@ int
 mag_initialize (MagEventCB mag_event)
 {
 	MagRectangle roi;
+	MagRectangle target_rect;
 
 	mag_zoomers_init ();
 	magnifier = get_magnifier ();
@@ -1493,17 +1526,20 @@ mag_initialize (MagEventCB mag_event)
 			 target_rect.width,
 			 target_rect.height);
 #endif
-        if (!srconf_set_data (MAGNIFIER_DISPLAY_SIZE_X, 
-			      CFGT_INT, 
-			      (gpointer)&target_rect.width, 
-			      MAGNIFIER_CONFIG_PATH)
-	) return FALSE;
+	display_size.left = display_size.top = DEFAULT_SCREEN_MIN_SIZE;
+	display_size.width = gdk_screen_width () - 1;
+	display_size.height = gdk_screen_height () - 1;
+	
+        srconf_set_data (MAGNIFIER_DISPLAY_SIZE_X, 
+			 CFGT_INT, 
+			 (gpointer)&display_size.width, 
+			 MAGNIFIER_CONFIG_PATH);
+	
 
-        if (!srconf_set_data (MAGNIFIER_DISPLAY_SIZE_Y, 
-			      CFGT_INT, 
-			      (gpointer)&target_rect.height, 
-			      MAGNIFIER_CONFIG_PATH)
-	) return FALSE;
+        srconf_set_data (MAGNIFIER_DISPLAY_SIZE_Y, 
+			 CFGT_INT, 
+			 (gpointer)&display_size.height, 
+			  MAGNIFIER_CONFIG_PATH);
 
 /*vertical split*/
 	 rect.top =  target_rect.top;
@@ -1513,8 +1549,6 @@ mag_initialize (MagEventCB mag_event)
 	
 	magnifier_clear_all_regions (magnifier);
 
-	magnifier_set_target (magnifier,
-			      &rect);
 	 roi.top =  0;
 	 roi.left = 0;
 	 roi.width = target_rect.width/2 ;
@@ -1538,5 +1572,3 @@ mag_terminate ()
 	fprintf(stderr,"\nTerminate the magnifier session\n");
 #endif
 }
-
-
