@@ -34,7 +34,7 @@
 
 #define SRC_PANNING_REASON_CARET	1
 #define SRC_PANNING_REASON_FOCUS	2
-#define SORTED_ON_COORD(a, b, c) ( (a >= 0 && a < b) ? ( (b < c) ? TRUE:FALSE):FALSE) 
+#define SORTED_ON_COORD(a, b, c) ( (a >= 0 && a < b) ? ( (b <= c) ? TRUE:FALSE):FALSE) 
 
 typedef struct
 {
