@@ -686,6 +686,37 @@ srl_get_focused_cell (Accessible *acc_table)
     return rv;
 }
 
+static Accessible*
+srl_get_focused_from_acc (Accessible *acc)
+{
+    Accessible *focused = NULL;
+    gint cnt, i;
+
+    srl_assert (acc);
+    
+    cnt = Accessible_getChildCount (acc);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child;
+	child = Accessible_getChildAtIndex (acc, i);
+	if (!child)
+	    continue;
+	if (srl_acc_has_state (child, SPI_STATE_FOCUSED))
+	{
+	    focused = child;
+	    Accessible_ref (focused);
+	}
+	else
+	    focused = srl_get_focused_from_acc (child);
+	
+	Accessible_unref (child);
+	if (focused)
+	    break;
+    }
+
+    return focused;
+}
+
 /* function returns the active child in a tree. The search
    is limited to maximum _limit_ number of children, children's chidren etc */
 static Accessible*
@@ -811,6 +842,12 @@ srle_set_acc (SRLEvent *event)
 	event->acc = srl_tree_get_active_item ((Accessible*)srle_get_acc (event), 
 						 &limit);
     }
+    else if (srle_has_type (event, SRL_EVENT_FOCUS) &&
+		srle_acc_has_role (event, SPI_ROLE_TABLE) &&
+		!srle_acc_has_state (event, SPI_STATE_MANAGES_DESCENDANTS))
+    {
+	event->acc = srl_get_focused_from_acc (srle_get_acc (event));
+    }
     else if (srle_has_type (event, SRL_EVENT_ACTIVE_DESCENDANT_CHANGED))
     {
 	event->acc = AccessibleActiveDescendantChangedEvent_getActiveDescendant (event->acc_ev);
@@ -1907,6 +1944,30 @@ srl_selection_event_is_reported (SRLEvent *event)
 static SRLLastInfo last_info_focus, last_info_watched;
 static gboolean srl_set_last_focus (Accessible *acc);
 
+static gboolean
+srle_is_parent_of_last_focus (SRLEvent *event)
+{
+    Accessible *parent;
+
+    if (!srl_last_focus)
+	return FALSE;
+
+    parent = srl_last_focus;
+    Accessible_ref (parent);
+    
+    while (parent && parent != srle_get_acc (event))
+    {
+	Accessible *tmp = parent;
+	parent = Accessible_getParent (parent);
+	Accessible_unref (tmp);
+    }
+    if (parent)
+    	Accessible_unref (parent);
+    
+    return parent != NULL;
+}
+
+
 static gboolean
 srl_event_is_reported (SRLEvent *event)
 {
@@ -2055,8 +2116,9 @@ srl_event_is_reported (SRLEvent *event)
     }
     if ((srle_has_type (event, SRL_EVENT_TAB_ADDED) || 
          srle_has_type (event, SRL_EVENT_TAB_REMOVED))) 
-	rv = srle_acc_has_role (event, SPI_ROLE_PAGE_TAB);
-	
+	rv = srle_acc_has_role (event, SPI_ROLE_PAGE_TAB) &&
+		!srle_is_parent_of_last_focus (event);
+
     /*In some situations we have two 'focus:' events for a page-tab 
      (an "object:state-changed:selected" event which was converted into
       a focus event _and_ a "focus:" event) => report a single "focus:" event*/	
@@ -2442,38 +2504,6 @@ srl_is_object_focused (SRObject *obj)
     return sro_get_acc (obj) == srl_last_focus || sro_get_acc (obj) == srl_last_focus2;
 }
 
-static Accessible*
-srl_get_focused_from_acc (Accessible *acc)
-{
-    Accessible *focused = NULL;
-    gint cnt, i;
-
-    srl_assert (acc);
-    
-    cnt = Accessible_getChildCount (acc);
-    for (i = 0; i < cnt; i++)
-    {
-	Accessible *child;
-	child = Accessible_getChildAtIndex (acc, i);
-	if (!child)
-	    continue;
-	if (srl_acc_has_state (child, SPI_STATE_FOCUSED))
-	{
-	    focused = child;
-	    Accessible_ref (focused);
-	}
-	else
-	    focused = srl_get_focused_from_acc (child);
-	
-	Accessible_unref (child);
-	if (focused)
-	    break;
-    }
-
-    return focused;
-}
-
-
 static Accessible*
 srl_get_focused_from_active_toplevel (Accessible *window)
 {
